(define-module (ordo play)
  #:use-module (srfi srfi-1) ; list utils
  #:use-module (srfi srfi-9) ; records
  #:use-module (srfi srfi-26) ; cut
  #:use-module (ordo connection)
  #:use-module (ordo context)
  #:use-module (ordo task)
  #:export (play run-play))

(define-record-type <play>
  (make-play description connection tasks handlers)
  play?
  (connection get-play-connection)
  (description get-play-description)
  (tasks get-play-tasks)
  (handlers get-play-handlers))

(define* (play description #:key connection tasks (handlers '()))
  (unless connection (error "connection is required"))
  (unless tasks (error "tasks are required"))
  (for-each (lambda (task)
              (for-each (lambda (trigger)
                          (unless (assoc-ref handlers trigger)
                            (error (format #f "task \"~a\" references an undefined trigger: ~a"
                                           (get-task-description task)
                                           trigger))))
                        (get-task-triggers task)))
            tasks)
  (make-play description connection tasks handlers))

(define (run-trigger conn ctx handlers trigger)
  (let ((handler (assoc-ref handlers trigger)))
    (unless handler
      (error (format #f "no handler defined for trigger ~a" trigger)))
    (run-task conn ctx handler)))

(define (run-play play)
  (format #t "Running play ~a~%" (get-play-description play))
  (call-with-connection
   (get-play-connection play)
   (lambda (c)
     (let ((tmp-dir (car (connection-must c "mktemp" '("--directory"))))
           (ctx (make-context)))
       (set-context-scratch-dir! ctx tmp-dir)
       (dynamic-wind
         (const #t)
         (lambda ()
           (for-each (cut run-task c ctx <>) (get-play-tasks play))
           (for-each (cut run-trigger c ctx (get-play-handlers play) <>)
                     (delete-duplicates (get-context-triggers ctx))))
         (lambda () (connection-must c "rm" `("-rf" ,tmp-dir))))))))
